/*
** VVZ v1.0.1
**
** A WWW lectures list
**
** Copyright (c) 11.4.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.0.1 - 16.3.95
**	Expand symbolic links as semester directory
**
** Version 1.0 - 11.4.94
**	Initial version
*/

char header[]="vvz v1.0.1\n(c) 1994 by Andreas Ley\n";

/* General definitions */

#define	TZ	"MEZ-1MESZ"
#define	MASTER	"webmaster@rz.uni-karlsruhe.de"
#define	REFERENCE	"<A HREF=\"/~Andreas.Ley/\"><EM>Andreas Ley</EM></A>"
#define	ICON	"/Icons/Logos/VVZsmall.gif"

/* File locations */

#define	ROOT	"/usr/local/etc/httpd/lib/vvz"
#define	LECTURES	"lectures"
#define	LINKS	"links"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <regex.h>
#include "vvz_german.h"

#ifdef NEEDSTRERROR
extern int	sys_nerr;
extern char	*sys_errlist[];
#define	strerror(errno)	((errno)>=sys_nerr?"Undefined error":sys_errlist[errno])
#endif /* NEEDSTRERROR */

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

#define	max(a,b)	((a)>(b)?(a):(b))
#define	abs(a)	((a)<0?-(a):(a))


/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);

/* Functions from wildmat.c */

extern int wildmat();

/* Global variables */

struct entry {
	struct entry 	*next;
	char		*id;
	char		*name;
	char		*text;
	char		*link;
	} *lectures=NULL;

char	root[]=ROOT;
char	master[]=MASTER;
char	*script_name;
FILE	*log;

char	*iso2din[256] = {
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	" ", "!", "\"", "#", "$", "%", "&", "'", 
	"(", ")", "*", "+", ",", "-", ".", "/", 
	"0", "1", "2", "3", "4", "5", "6", "7", 
	"8", "9", ":", ";", "<", "=", ">", "?", 
	"@", "a", "b", "c", "d", "e", "f", "g", 
	"h", "i", "j", "k", "l", "m", "n", "o", 
	"p", "q", "r", "s", "t", "u", "v", "w", 
	"x", "y", "z", "[", "\\", "]", "^", "_", 
	"`", "a", "b", "c", "d", "e", "f", "g", 
	"h", "i", "j", "k", "l", "m", "n", "o", 
	"p", "q", "r", "s", "t", "u", "v", "w", 
	"x", "y", "z", "{", "|", "}", "~", NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	" ", "!", "c", "l", "c", "y", "|", "#", 
	NULL, "c", "a", "<", "-", "-", "r", "_", 
	"o", "+", "2", "3", NULL, "u", "p", ".", 
	NULL, "1", "o", ">", "1/4", "1/2", "3/4", "?", 
	"a", "a", "a", "a", "ae", "a", "ae", "c", 
	"e", "e", "e", "e", "i", "i", "i", "i", 
	"d", "n", "o", "o", "o", "o", "oe", "x", 
	"o", "u", "u", "u", "ue", "y", "p", "ss", 
	"a", "a", "a", "a", "ae", "a", "ae", "c", 
	"e", "e", "e", "e", "i", "i", "i", "i", 
	"d", "n", "o", "o", "o", "o", "oe", "/", 
	"o", "u", "u", "u", "ue", "y", "p", "y",
	};

char *iso2html[256]= {
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, "\t", "\n", NULL, NULL, "\r", NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	" ", "!", "\"", "#", "$", "%", "&amp;", "'", 
	"(", ")", "*", "+", ",", "-", ".", "/", 
	"0", "1", "2", "3", "4", "5", "6", "7", 
	/*
	"8", "9", ":", ";", "&lt;", "=", "&gt;", "?", 
	*/
	"8", "9", ":", ";", "<", "=", ">", "?", 
	"@", "A", "B", "C", "D", "E", "F", "G", 
	"H", "I", "J", "K", "L", "M", "N", "O", 
	"P", "Q", "R", "S", "T", "U", "V", "W", 
	"X", "Y", "Z", "[", "\\", "]", "^", "_", 
	"`", "a", "b", "c", "d", "e", "f", "g", 
	"h", "i", "j", "k", "l", "m", "n", "o", 
	"p", "q", "r", "s", "t", "u", "v", "w", 
	"x", "y", "z", "{", "|", "}", "~", " ", 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
	" ", "!`", "cents", "pound", "(currency)", "yen", "|", "(sect)", 
	"\"", "(C)", "na.", "&lt;&lt;", "(neg)", "-", "(R)", "=", 
	"^o", "+-", "^2", "^3", "'", "mu", "(par)", "*", 
	",", "^1", "no.", "&gt;&gt;", "1/4", "1/2", "3/4", "?`", 
	"&Agrave;", "&Aacute;", "&Acirc;", "&Atilde;", "&Auml;", "&Aring;", "&AElig;", "&Ccedil;", 
	"&Egrave;", "&Eacute;", "&Ecirc;", "&Euml;", "&Igrave;", "&Iacute;", "&Icirc;", "&Iuml;", 
	"&ETH;", "&Ntilde;", "&Ograve;", "&Oacute;", "&Ocirc;", "&Otilde;", "&Ouml;", "x", 
	"&Oslash;", "&Ugrave;", "&Uacute;", "&Ucirc;", "&Uuml;", "&Yacute;", "&THORN;", "&szlig;", 
	"&agrave;", "&aacute;", "&acirc;", "&atilde;", "&auml;", "&aring;", "&aelig;", "&ccedil;", 
	"&egrave;", "&eacute;", "&ecirc;", "&euml;", "&igrave;", "&iacute;", "&icirc;", "&iuml;", 
	"&eth;", "&ntilde;", "&ograve;", "&oacute;", "&ocirc;", "&otilde;", "&ouml;", "/", 
	"&oslash;", "&ugrave;", "&uacute;", "&ucirc;", "&uuml;", "&yacute;", "&thorn;", "&yuml;", 
	};



void stradd(char **dest, char *src)
{
	if (src)
		while (*src)
			*(*dest)++=*src++;
}



char *din(char *src)
{
	static char	buffer[BUFSIZ];
	char		*dest;

	dest=buffer;
	while (*src)
		stradd(&dest,iso2din[(unsigned char)*src++]);
	*dest='\0';
	return(buffer);
}



char *html(char *src)
{
	static char	buffer[BUFSIZ];
	char		*dest;

	dest=buffer;
	while (*src)
		stradd(&dest,iso2html[(unsigned char)*src++]);
	*dest='\0';
	return(buffer);
}



int strptrcmp(const void *s1, const void *s2)
{
	return(strcmp(*(char **)s1,*(char **)s2));
}



char *semester(char *path)
{
	static char	buffer[32];

	if (*path=='S')
		(void)sprintf(buffer,"Sommersemester '%d",atoi(path+2)%100);
	else
		(void)sprintf(buffer,"Wintersemester '%d/'%d",atoi(path+2)%100,(atoi(path+2)+1)%100);
	return(buffer);
}



/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *title, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"",title?" - ":"",title?title:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",MASTER);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	if (title)
		(void)printf("<EM>%s%s%s</EM><HR>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
#ifdef ICON
	(void)printf("<H1>%s <IMG SRC=\"" ICON "\" ALIGN=MIDDLE ALT=\"\"></H1>\n",title?title:TITLE);
#else
	(void)printf("<H1>%s</H1>\n",title?title:TITLE);
#endif
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	/*
	(void)printf("<P>%s\n",FOOTER);
	(void)printf("<ISINDEX>\n");
	*/
	(void)printf(DISCLAIMER,master);
	(void)printf("\n%s\n",REFERENCE);
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(NULL,ERROR,error);
	show_trailer();
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(NULL,ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer();
	exit(0);
}



int get_lectures()
{
	char		buffer[BUFSIZ],*ptr,*nptr;
	FILE		*src;
	struct entry	**next,*lecture;

	next=&lectures;
	if (!(src=fopen(LECTURES,"r")))
		return(FALSE);
	while(fgets(buffer,sizeof(buffer),src)) {
		ptr=strchr(buffer,'\0');
		if (ptr>buffer&&ptr[-1]=='\n')
			ptr[-1]='\0';
		*next=(struct entry *)malloc(sizeof (struct entry));
		ptr=strchr(buffer,'\t');
		*ptr++='\0';
		(*next)->id=strdup(buffer);
		nptr=strchr(ptr,'\t');
		*nptr++='\0';
		(*next)->name=strdup(ptr);
		(*next)->text=strdup(nptr);
		(*next)->link=NULL;
		next=&(*next)->next;
	}
	(void)fclose(src);
	*next=NULL;

	if (!(src=fopen(LINKS,"r")))
		return(FALSE);
	while(fgets(buffer,sizeof(buffer),src)) {
		ptr=strchr(buffer,'\0');
		if (ptr>buffer&&ptr[-1]=='\n')
			ptr[-1]='\0';
		if (!(ptr=strchr(buffer,'\t')))
			if (!(ptr=strchr(buffer,' ')))
				show_fatal(ERROR_BAD_FORMAT);
		*ptr++='\0';
		for (lecture=lectures;lecture&&strcmp(lecture->id,buffer);lecture=lecture->next);
		if (!lecture)
			show_fatal(ERROR_BAD_LINK);
		lecture->link=strdup(ptr);
	}
	(void)fclose(src);
	return(TRUE);
}



char *format(char *raw)
{
	static char	buffer[BUFSIZ];
	char		*src,*dest;

	for (src=raw;*src&&*src=='*';src++);
	for (dest=buffer;*src&&*src!='\t';src++,dest++)
		*dest=*src;
	if (*src++) {
		*dest++=',';
		*dest++=' ';
		for (;*src&&*src!='\t';src++,dest++)
			*dest=*src;
		if (*src++) {
			*dest++=',';
			*dest++=' ';
			if ('0'<=*src&&*src<='9')
				*dest++='N';
			for (;*src&&*src!='\t';src++,dest++)
				*dest=*src;
		}
	}
	*dest='\0';
	return(buffer);
}



/* Displays the main menu */
void main_menu()
{
	int		cnt;
	FILE		*src;
char	*entry[1024];
	char		buffer[BUFSIZ];
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;
int	max;

	if ((dir=opendir("."))) {
		max=0;
		while ((dp=readdir(dir))) {
			if (strlen(dp->d_name)==6
				&&(!strncmp(dp->d_name,"WS",2)||!strncmp(dp->d_name,"SS",2))) {
					(void)stat(dp->d_name,&buf);
					if (S_ISDIR(buf.st_mode)) {
						sprintf(buffer,"%s%c%s",dp->d_name+2,*dp->d_name=='S'?'0':'1',dp->d_name);
						entry[max++]=strdup(buffer);
					}
			}
		}
		(void)closedir(dir);
		if (max)
			qsort((void *)entry,max,sizeof(char *),strptrcmp);
		else
			show_fatal(ERROR_EMPTY);
	}
	else
		show_fatal(ERROR_DIRECTORY);

	show_header(NULL,NULL,MAIN_MENU_HEADER);
	printf("<BLOCKQUOTE>\n");
	for (cnt=0;cnt<max;cnt++)
		(void)printf("<A HREF=\"%s/S%s\">%s</A><BR>\n",script_name,entry[cnt]+5,semester(entry[cnt]+5));
	printf("</BLOCKQUOTE>\n");
	show_trailer();
}


void semester_menu(char *path)
{
	int		len,cnt;
	FILE		*src;
	char		*entry[1024];
	char		buffer[BUFSIZ],*ptr;
	DIR		*dir;
	struct dirent	*dp;
	int		max;

	if ((len=readlink(path,buffer,sizeof(buffer)))>0) {
		buffer[len]='\0';
		path=strdup(buffer);
	}
	(void)chdir(path);
	if ((dir=opendir("."))) {
		max=0;
		while ((dp=readdir(dir)))
			if ('0'<=dp->d_name[0]&&dp->d_name[0]<='9')
					entry[max++]=strdup(dp->d_name);
		(void)closedir(dir);
		if (max)
			qsort((void *)entry,max,sizeof(char *),strptrcmp);
		else
			show_fatal(ERROR_EMPTY);
	}
	else
		show_fatal(ERROR_DIRECTORY);

	show_header(NULL,semester(path),SEMESTER_MENU_HEADER);
	printf("<BLOCKQUOTE>\n");
	for (cnt=0;cnt<max;cnt++) {
		if ((src=fopen(entry[cnt],"r"))) {
			(void)fgets(buffer,sizeof(buffer),src);
			(void)fclose(src);
		}
		else
			show_fatal(ERROR_READ);
		ptr=strchr(buffer,'\0');
		if (ptr>buffer&&ptr[-1]=='\n')
			ptr[-1]='\0';
		(void)printf("<A HREF=\"%s/F%s/%s\">%s</A><BR>\n",script_name,path,entry[cnt],html(buffer+1));
	}
	printf("</BLOCKQUOTE>\n");
	show_trailer();
}



void faculty_menu(char *path)
{
	int		len,line,anchor,empty;
	FILE		*src;
	char		buffer[BUFSIZ],*fname,*ptr,*nptr,c;
	enum		{HEADER,LECTURE,COMMENT,BETWEEN} comment;
	struct entry	*lecture;

	anchor=0;
	comment=HEADER;
	empty=0;
	fname=strchr(path,'/');
	*fname++='\0';
	if ((len=readlink(path,buffer,sizeof(buffer)))>0) {
		buffer[len]='\0';
		path=strdup(buffer);
	}
	(void)chdir(path);
	get_lectures();
	if (ptr=strchr(fname,'/')) {
		*ptr++='\0';
		anchor=atoi(ptr);
	}
	if (src=fopen(fname,"r")) {
		line=0;
		fgets(buffer,sizeof(buffer),src);
		ptr=strchr(buffer,'\0');
		if (ptr>buffer&&ptr[-1]=='\n')
			ptr[-1]='\0';
		show_header(semester(path),html(buffer+1),LIST_HEADER);
		while(fgets(buffer,sizeof(buffer),src)) {
			line++;
			ptr=strchr(buffer,'\0');
			if (ptr>buffer&&ptr[-1]=='\n')
				ptr[-1]='\0';
			switch (*buffer) {
			case 'H':
				if (comment==COMMENT||comment==BETWEEN)
					(void)printf("</EM>\n");
				comment=HEADER;
				(void)printf("<H3>%s</H3>\n",html(buffer+1));
				break;
			case 'C':
				if (buffer[1]) {
					if (comment==HEADER)
						(void)printf("<EM>");
					else if (comment==LECTURE)
						(void)printf("<P>\n<EM>");
					else if (comment==BETWEEN)
						(void)printf("<P>\n");
					else
						(void)printf("<BR>\n");
					comment=COMMENT;
					(void)printf("%s",html(buffer+1));
				}
				else {
					if (comment==COMMENT)
						comment=BETWEEN;
					empty++;
				}
				break;
			default:
				if (comment==COMMENT||comment==BETWEEN)
					(void)printf("</EM><P>\n");
				if (comment==LECTURE)
					(void)printf("<BR>");
				comment=LECTURE;
				for (lecture=lectures;lecture&&strcmp(lecture->id,buffer);lecture=lecture->next);
				if (!lecture)
					show_fatal(ERROR_NO_LECTURE);
				strcpy(buffer,lecture->name);
				if ((ptr=strchr(buffer,'('))) {
					while (ptr>buffer&&ptr[-1]==' ')
						ptr--;
					c=*ptr;
					*ptr++='\0';
				}
				if (lecture->link)
					(void)printf("<A HREF=\"%s\">%s</A>",lecture->link,lecture->id);
				else
					(void)printf("%s",lecture->id);
				(void)printf(" <A HREF=\"%s/L%s/%s\"",script_name,path,lecture->id);
				fflush(stdout);
				if (line==anchor)
					(void)printf(" NAME=\"*\"><STRONG");
				(void)printf(">%s",html(buffer));
				if (line==anchor)
					(void)printf("</STRONG>");
				(void)printf("</A>");
				if (ptr)
					(void)printf("%c%s",c,html(ptr));
				(void)printf("\n");
			}
		}
		(void)fclose(src);
		show_trailer();
	}
	else
		show_fatal(ERROR_READ);
}



char *strong(char *text, int len)
{
	static char	buffer[BUFSIZ];

	strcpy(buffer,"<STRONG>");
	strcat(buffer,text);
	strcat(buffer,"</STRONG>");
	len-=strlen(text);
	while (len--)
		strcat(buffer," ");
	return(buffer);
}



void show(char *path)
{
	FILE	*src;
	char	buffer[BUFSIZ],hdr[BUFSIZ],*field[BUFSIZ/2],reference[6];
	char	*id,*ptr,*nptr;
	struct entry	*lecture,*tlecture,*llecture;
	int	f,fields,l1,l2,len;

	id=strchr(path,'/');
	*id++='\0';
	if ((len=readlink(path,buffer,sizeof(buffer)))>0) {
		buffer[len]='\0';
		path=strdup(buffer);
	}
	(void)chdir(path);
	get_lectures();

	for (lecture=lectures;lecture&&strcmp(lecture->id,id);lecture=lecture->next);
	if (!lecture)
		show_fatal(ERROR_NO_LECTURE);

	ptr=strchr(lecture->text,'\0');
	if (ptr>lecture->text&&ptr[-1]=='\n')
		ptr[-1]='\0';
	for (ptr=lecture->text,fields=2;ptr;) {
		field[fields++]=ptr;
		ptr=strchr(ptr,'\t');
		if (ptr)
			*ptr++='\0';
	}

	/* Look for a link in the name to another lecture */
	strcpy(buffer,lecture->name);
	/* Numerical link? */
	for (ptr=buffer;strlen(ptr)>=5;ptr++)
		if ('0'<=ptr[0]&&ptr[0]<='9'
			&&'0'<=ptr[1]&&ptr[1]<='9'
			&&'0'<=ptr[2]&&ptr[2]<='9'
			&&'0'<=ptr[3]&&ptr[3]<='9'
			&&'0'<=ptr[4]&&ptr[4]<='9')
			break;
	/* Found numerical link */
	if (strlen(ptr)>=5) {
		strncpy(reference,ptr,5);
		reference[5]='\0';
		*ptr='\0';
		sprintf(hdr,"%s<A HREF=\"%s/L%s/%s\">%s</A>",html(buffer),script_name,path,reference,reference);
		strcat(hdr,html(ptr+5));
	}
	/* Look for a different lecture whose name is a substring of the current */
	/* Patch: check that the first two digits match */
	else {
		for (llecture=NULL,tlecture=lectures;tlecture;tlecture=tlecture->next)
			if (strcmp(tlecture->id,id)&&!strncmp(tlecture->id,id,2)&&(nptr=strstr(buffer+1,tlecture->name))&&(!llecture||strlen(tlecture->name)>strlen(llecture->name))) {
					llecture=tlecture;
					ptr=nptr;
			}
		if (llecture) {
			*ptr='\0';
			strcpy(hdr,html(buffer));
			sprintf(strchr(hdr,'\0'),"<A HREF=\"%s/L%s/%s\">%s</A>",script_name,path,llecture->id,html(llecture->name));
			strcat(hdr,html(ptr+strlen(llecture->name)));
		}
		else
			strcpy(hdr,html(buffer));
	}
	if ((ptr=strchr(hdr,'('))) {
		nptr=ptr;
		while (nptr>hdr&&nptr[-1]==' ')
			nptr--;
		*nptr='\0';
	}
	show_header(semester(path),hdr,SHOW_HEADER);
	if (ptr)
		printf("(%s\n",ptr+1);

	len=0;
	if (*field[2])
		len=strlen(strchr(field[2],',')||strstr(field[2],AND)||strstr(field[2],WITH)?F2P:F2);
	if (*field[3])
		len=max(len,strlen(F3));
	if (fields>4)
		len=max(len,strlen(fields>8?F4P:F4));
	len++;

	printf("<PRE>");
	printf("%s%s\n",strong(F0,len),lecture->id);
	if (*field[2])
		printf("%s%s\n",strong(strchr(field[2],',')||strstr(field[2],AND)||strstr(field[2],WITH)?F2P:F2,len),field[2]);
	if (*field[3])
		printf("%s%s\n",strong(F3,len),field[3]);
	if (fields>4) {
		printf("%s",strong(fields>11?F4P:F4,len));
		for(l1=0,l2=0,f=4;f<fields;f+=4) {
			l1=max(l1,strlen(field[f+1]));
			l2=max(l2,strlen(field[f+2]));
		}
		for(f=4;f+3<fields;f+=4) {
			printf("%*s%*s%*s%s",f>4?len:0,"",-l1-1,field[f+1],-l2-(l2?1:0),field[f+2],field[f]);
			if (*field[f+3])
				printf("%s(%s)",*field[f]?" ":"",field[f+3]);
			printf("\n");
		}
	}
	printf("</PRE>\n");
	if (lecture->link) {
		(void)printf(SHOW_MORE_INFO,lecture->link);
	}
	show_trailer();
}



/* Process search request */
void search(char *pattern)
{
	int	cnt,line;
	FILE	*src;
char	*entry[1024];
	char	header[BUFSIZ],buffer[BUFSIZ],*ptr;
int	max=0;

	show_header(NULL,NULL,SEARCH_HEADER);
	for (cnt=0;cnt<max;cnt++)
		if (src=fopen(entry[cnt],"r")) {
			line=0;
			fgets(header,sizeof(header),src);
			if (wildmat(header,pattern))
				(void)printf("<A HREF=\"%s/%s\">%s</A><BR>\n",script_name,entry[cnt],header);
			while(fgets(buffer,sizeof(buffer),src)) {
				line++;
				if (wildmat(buffer,pattern))
					(void)printf("<A HREF=\"%s/%s/%d#*\">%s</A> - <A HREF=\"%s/%s\">%s</A><BR>\n",script_name,entry[cnt],line,format(buffer),script_name,entry[cnt],header);
			}
			(void)fclose(src);
		}
	show_trailer();
}




void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


/*
** CGI Script beeing called with the folloing path_info arguments:
** First character is command:
** S Semester menu, followed by the selected semester directory
** F Faculty menu, followed by the selected semester and faculty file
**   May be followed by a line number of a search request
** L Lecture menu, followed by the selected semester and lecture number
** Q Query
** The list of selected lectures followes as three-character-codes after a comma
*/
int main(int argc, char *argv[])
{
	int		c;
	extern int	optind;
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;
	char		*path_info,*query_string,*ptr,*nptr,*q,*p;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	if (strcmp(getenv("REQUEST_METHOD"),"GET"))
		show_fatal(ERROR_REQUEST_METHOD);

	(void)putenv("TZ=" TZ);
	script_name=getenv("SCRIPT_NAME");
	path_info=getenv("PATH_INFO");
	if (*path_info=='/')
		path_info++;
	if ((ptr=strchr(path_info,','))) {
		*ptr='\0';
	}
	if (!*path_info)
		path_info="@";
	query_string=getenv("QUERY_STRING");

	ptr=malloc(strlen(query_string)+3);
	p=ptr;
	q=query_string;
	if (q[0]!='*')
		*p++='*';
	for (;*q;p++,q++)
		*p=*q=='+'?'*':*q;
	if (p>ptr&&p[-1]!='*')
		*p++='*';
	*p='\0';

	(void)chdir(root);
	log=fopen("/tmp/vvz.log","w");
	if (*query_string)
		show_fatal(ERROR_NOT_YET_IMPLEMENTED);
	else
		switch (*path_info++) {
		case 'S':
			semester_menu(path_info);
			break;
		case 'F':
			faculty_menu(path_info);
			break;
		case 'L':
			show(path_info);
			break;
		default:
			main_menu();
		}

	fclose(log);
	return(0);
}
